<cfcomponent extends="components.persistence.basepersistentcomponent">
	<!--- Properties --->
	<cfproperty name="maskInfo" type="string" default="">
	<cfproperty name="isMask" type="boolean" default="false">
	<cfproperty name="contentLocale" type="string" default="en_US">
	<cfproperty name="status" type="boolean">
	<!--- Constructor --->
	<cfscript>
		this.isMask = "false";
		this.status = 0;
		this.contentLocale = "en_US";
	</cfscript>
	<!--- Get/Set functions --->
	<cffunction access="public" name="getStatus" output="false" returntype="numeric">
		<cfreturn this.status/>
	</cffunction>
	<cffunction access="public" name="setStatus" output="false">
		<cfargument name="theStatus" type="boolean" required="true">
		<cfset this.status = arguments.theStatus>
	</cffunction>
	<cffunction name="getContentLocale" access="public" output="false" returntype="string">
		<cfreturn this.contentLocale/>
	</cffunction>
	<cffunction name="setContentLocale" access="public" output="false">
		<cfargument name="theLocale" type="string" required="true">
		<cfset this.contentLocale = arguments.theLocale>
	</cffunction>
	<cffunction name="getIsMask" access="public" output="false" returntype="boolean">
		<cfreturn this.isMask/>
	</cffunction>
	<cffunction name="setIsMask" access="public" output="false">
		<cfargument name="theMask" type="boolean" required="true">
		<cfset this.isMask = arguments.theMask>
	</cffunction>
	<cffunction name="getMaskInfo" access="public" output="false" returntype="string">
		<cfreturn this.maskInfo/>
	</cffunction>
	<cffunction name="setMaskInfo" access="public" output="false">
		<cfargument name="themaskinfo" type="string" required="true">
		<cfset this.maskinfo = arguments.themaskinfo>
	</cffunction>
	<!--- createMask(): This function creates a primary mask object for a new piece of content 
						in the passed locale. A MaskInfo instance is also created to store
						all localization data for this content.
	--->
    <cffunction name="createMask"
		access="public"
        returntype="any"
        output="false">
        <cfargument name="contentLocale" type="string" required="true"/>
        <cfscript>
		    this.isMask = "true";
		    this.contentLocale = arguments.contentLocale;
            this.status = 1;
		    // Create a MaskInfo instance to store the locale
            // information for all locales of this object  
            myMaskInfo = createObject("component","MaskInfo");
		    myMaskInfo.setActiveLocales(arguments.contentLocale);
		    myMaskInfo.setAvailableLocales(arguments.contentLocale);
		    myMaskInfo.create();
		    this.maskInfo = myMaskInfo.objectID;
            // Now create this mask object
            this.create();
		    return this;
        </cfscript>
    </cffunction>
    <!--- createLocale() - This function creates a new localized instance of the object
          					using the mask object's ID and appending the locale for 
							this instance --->
    <cffunction name="createLocale"
        access="public"
        returntype="any"
        output="false">
        <cfargument name="contentLocale" type="string" required="true"/>
        <!--- Validate that this is the mask object --->
        <cfif not this.isMask>
            <cfthrow message="Validation error">
        </cfif>
        <!--- Load the MaskInfo helper class --->
        <cfscript>
            myMaskInfo = createObject("component","MaskInfo");
            myMaskInfo.get("database", "testDB", this.maskInfo);
        </cfscript>
        <!--- Validate that no object has been created for the passed locale --->
        <cfif listFindNoCase(myMaskInfo.getAvailableLocales(),arguments.contentLocale)>
            <cfthrow message="Validation error">
        </cfif>
        <cfscript>
		    newLocale = createObject("component",getMetadata(this).name);
            // Copy props into this new locale object
            for( prop in this ) {
                if( not isCustomFunction(this[prop]) )
                    newLocale[prop] = this[prop];
            }
		    // Over-ride necessary properties
            newLocale.setContentLocale(arguments.contentLocale);
            newLocale.setIsMask(false);
            newLocale.setStatus(1); 
            // Set up the ID to use when creating this new 
            // locale instance
            localeID = this.objectID & "/" & arguments.contentLocale; 
            // Create the new locale instance passing in
            // the desired object ID
            newLocale.create(objectID=localeID); 
            // Update the MaskInfo object
            myMaskInfo.addActiveLocale(arguments.contentLocale); 
            myMaskInfo.addAvailableLocale(arguments.contentLocale);
            myMaskInfo.update();
            // Return the new locale instance
		    return newLocale;
        </cfscript>
    </cffunction> 
    <!--- Function: getLocales() --->
    <cffunction name="getLocales"
        access="public"
        returntype="string"
        output="false">
        <cfargument name="requestedLocales" type="string" required="false"/>
        <!--- Validate that this is the mask object --->
        <cfif not this.isMask>
            <cfthrow message="Validation error">
        </cfif>
		<cfoutput><strong>#this.maskinfo#</strong></cfoutput>
        <cfscript>
            myMaskInfo = createObject("component","MaskInfo");
       	    myMaskInfo.get("database", "testDB", this.maskInfo);
		</cfscript>
		<cfscript>
       	    // Set the localeType if not passed
            if( not structKeyExists(arguments,"requestedLocales") )
         		arguments.requestedLocales = "available";
       		// Return the locales the user wants
			switch (arguments.requestedLocales) {
         		case "active":
           			return myMaskInfo.getActiveLocales();
           			break;
         		case "available":
           			return myMaskInfo.getAvailableLocales();
           			break;
         		default:
           			return myMaskInfo.getAvailableLocales();
           			break;
       		}
     	</cfscript>
    </cffunction> 
    <!--- Function: deleteMask() --->
    <cffunction name="deleteMask"
        access="public"
        returntype="boolean"
        output="false">
        <!--- Validate that this is the mask object --->
        <cfif not this.isMask>
            <cfthrow message="Validation error">
        </cfif> 
        <cfscript>
	        localesToDelete = this.getLocales();
		    numLocales = listLen(localesToDelete);
            if( numLocales GT 1 ) {
		        tempLocale = createObject("component",getMetadata(this).name);         
                for(i = 1; i lte numLocales; i = i + 1) {
            		theLocale = listGetAt(localesToDelete,i);
            		if(theLocale NEQ this.contentLocale) { 
             			theID = this.objectID & "/" & theLocale;
		      			tempLocale = tempLocale.get("database", "contentDB", theID);
             			tempLocale.delete();
           			}
         		}
       		}
       		this.delete();
       		return true;
     	</cfscript>
   	</cffunction>            	
</cfcomponent>