<!---
AUTHOR: Anthony C. McClure
DATE: 4/25/2002
PURPOSE: Main persistence component.  All component objects extend this component.  
		Contains the basic API for object interaction.
MODIFICATION LOG:
DATE        AUTHOR      MODIFICATION
====        =======     ============
4/25/2002	ACM			Initial Generation

--->

<cfcomponent output="FALSE">
	<cfproperty name="objectID" type="UUID" default="">
	<cfproperty name="typeID" type="string" default="">
	<cfproperty name="bActive" type="boolean" default="1">
	<cfproperty name="createdBy" type="string" default="">
	<cfproperty name="dtCreatedDate" type="date" default="">
	<cfproperty name="dtLastUpdatedDate" type="date" default="">
	<cfproperty name="lastUpdatedBy" type="string" default="">
	<cfproperty name="dbPersistor" default="">
	
	<!--- Initialize Persistors and Properties--->
	<cfscript>
		this.dbPersistor = createObject("component","dbPersistor");
		
		this.objectID = createUUID();
		this.typeID = "";
		this.bActive = "1";
		this.createdBy = "";
		this.dtCreatedDate = createODBCDate(Now());
		this.dtLastUpdatedDate = createODBCDate(Now());
		this.lastUpdatedBy = "";
	</cfscript>

	<!--- Property Gets and Sets Start --->
	<cffunction returnType="UUID" access="public" name="getObjectID" output="FALSE">
		<cfreturn this.objectID>
	</cffunction>
	<cffunction access="public" name="setObjectID" output="FALSE">
		<cfargument name="theObjectID" type="UUID" required="1"/>
		<cfset this.objectID = arguments.theObjectID>
	</cffunction>
	<cffunction returnType="string" access="public" name="getTypeID" output="FALSE">
		<cfreturn this.typeID>
	</cffunction>
	<cffunction access="public" name="setTypeID" output="FALSE">
		<cfargument name="theTypeID" type="string" required="1"/>
		<cfset this.typeID = arguments.theTypeID>
	</cffunction>
	<cffunction returnType="boolean" access="public" name="getBActive" output="FALSE">
		<cfreturn this.bActive>
	</cffunction>
	<cffunction access="public" name="setBActive" output="FALSE">
		<cfargument name="theBActive" type="boolean" required="1"/>
		<cfset this.bActive = arguments.thebActive>
	</cffunction>
	<cffunction returnType="string" access="public" name="getCreatedBy" output="FALSE">
		<cfreturn this.createdBy>
	</cffunction>
	<cffunction access="public" name="setCreatedBy" output="FALSE">
		<cfargument name="theCreatedBy" type="string" required="1"/>
		<cfset this.createdBy = arguments.theCreatedBy>
	</cffunction>
	<cffunction returnType="string" access="public" name="getDtCreatedDate" output="FALSE">
		<cfreturn this.dtCreatedDate>
	</cffunction>
	<cffunction access="public" name="setDtCreatedDate" output="FALSE">
		<cfargument name="theDtCreatedDate" type="string" required="1"/>
		<cfset this.dtCreatedDate = arguments.theDtCreatedDate>
	</cffunction>
	<cffunction returnType="string" access="public" name="getDtLastUpdatedDate" output="FALSE">
		<cfreturn this.dtLastUpdatedDate>
	</cffunction>
	<cffunction access="public" name="setDtLastUpdatedDate" output="FALSE">
		<cfargument name="theDtLastUpdatedDate" type="string" required="1"/>
		<cfset this.dtLastUpdatedDate = arguments.theDtLastUpdatedDate>
	</cffunction>
	<cffunction returnType="string" access="public" name="getLastUpdatedBy" output="FALSE">
		<cfreturn this.lastUpdatedBy>
	</cffunction>
	<cffunction access="public" name="setLastUpdatedBy" output="FALSE">
		<cfargument name="theLastUpdatedBy" type="string" required="1"/>
		<cfset this.lastUpdatedBy = arguments.theLastUpdatedBy>
	</cffunction>
	<!--- Property Gets and Sets End --->
	
	<!--- Persistence API Start --->
	<cffunction access="public" name="deploy" output="0" returnType="boolean">
		<!--- The deployment function, creates a new object type in the content repositories.
		The datastoretype argument is required and must be one of the switch values.  If "database" is selected
		as the datastoretype, then datasource is required as well. --->
		<cfargument name="dataStoreType" type="string" required="1"/>
		<cfargument name="dataSource" type="string" required="0" default="testDB"/>
		
		<cfscript>
			switch (arguments.dataStoreType) {
				case "database":
				{
					returnVal = this.dbPersistor.installObjectType(this,arguments.dataSource);	
					break;
				}
			}
			
			if (returnVal)
				return true;
			else
				return false;
		</cfscript>
	</cffunction>
	<cffunction access="public" name="remove" output="FALSE">
		<!--- The remove function takes all reference out of the content repositories for the given
		type.  This function CANNOT be undone. --->
	</cffunction>
	<cffunction access="public" name="create" returnType="boolean" output="0">
		<!--- This function persists a new object into the content datastore.  The datastoretype argument is 
		optional and will default to database if not provided. --->
		<cfargument name="dataStoreType" type="string" required="0" default="database"/>
		<cfargument name="dataSource" type="string" required="0" default="testDB"/>
		
		<cfscript>
			switch (arguments.dataStoreType) {
				case "database":
				{
					returnVal = this.dbPersistor.createPersistenceObject(this,arguments.dataSource);	
					break;
				}
			}
			
			if (returnVal)
				return true;
			else
				return false;
		</cfscript>
	</cffunction>
	<cffunction access="public" name="update" returnType="boolean" output="FALSE">
		<!--- The update function persists changes to an object to the content repository.  The datastoretype 
		argument is optional and will default to database if not provided.  Returns a boolean success result. --->
		<cfargument name="dataStoreType" type="string" required="0" default="database"/>
		<cfreturn true>
	</cffunction>
	<cffunction access="remote" name="get" output="1">
		<!--- The get function acts as the class initializer.  It will set the data into the CFC properties
		for the passed in object and typeID's.  Once again, dataStoreType is optional and will 
		default to database. --->
		<cfargument name="dataStoreType" type="string" required="0" default="database"/>
		<cfargument name="dataSource" type="string" required="0" default="testDB"/>
		<cfargument name="objectID" type="UUID" required="1">
		
		<cfscript>
			switch (arguments.dataStoreType) {
				case "database":
				{
					returnVal = this.dbPersistor.getPersistenceObject(this,arguments.dataSource,arguments.objectID);	
					break;
				}
			}
			
			if (returnVal)
				return true;
			else
				return false;
		</cfscript>
	</cffunction>
	<cffunction access="public" name="delete" output="FALSE">
		<!--- The delete function removes the object instance from the persistence datastore. --->
		<cfargument name="dataStoreType" type="string" required="0" default="database"/>
	</cffunction>
	<cffunction access="public" name="dumpProperties" output="0" returnType="struct">
		<cfset objectMetaData = getMetadata(this)>

		<!--- Place all properties into a single array --->
		<cfscript>
			aAllProperties = arrayNew(1);
			for(i=1;i LTE arrayLen(objectMetaData.properties);i = i + 1) {
				arrayAppend(aAllProperties,objectMetaData.properties[i]);
			}
			for(i=1;i LTE arrayLen(objectMetaData.extends.properties);i = i + 1) {
				arrayAppend(aAllProperties,objectMetaData.extends.properties[i]);
			}
			
			stReturn = structNew();
			for (i=1;i LTE arrayLen(aAllProperties);i = i + 1) {
				if (structKeyExists(aAllProperties[i],"type")) {
					evalThisProperty = "this.get" & aAllProperties[i].name & "()";
					structInsert(stReturn,aAllProperties[i].name,evaluate(evalThisProperty));
				}
			}
		</cfscript>
		
		<cfreturn stReturn>
	</cffunction>
	<!--- Persistence API End --->
</cfcomponent>